import svelte from "rollup-plugin-svelte";
import resolve from "@rollup/plugin-node-resolve";
import commonjs from "@rollup/plugin-commonjs";
import livereload from "rollup-plugin-livereload";
import { terser } from "rollup-plugin-terser";
import autoPreprocess from "svelte-preprocess";
import typescript from "@rollup/plugin-typescript";
import postcss from "rollup-plugin-postcss";

const production = !process.env.ROLLUP_WATCH;

const rollupConfigFor = (name, demo = false) =>
  (production && !demo) || (!production && demo)
    ? [
        {
          input: `src/${name}/index.js`,
          output: {
            sourcemap: true,
            format: "iife",
            name: [...name].filter((char) => char !== "-").join(""),
            file: `public/build/${name}/index.js`,
          },
          plugins: [
            typescript(),
            postcss({
              minimize: true,
              use: [
                [
                  "sass",
                  {
                    includePaths: ["./src/theme", "./node_modules"],
                  },
                ],
              ],
            }),
            svelte({
              customElement: !demo,
              // enable run-time checks when not in production
              dev: !production,
              // we'll extract any component CSS out into
              // a separate file - better for performance
              css: (css) => {
                css.write(`public/build/${name}/style.css`);
              },
              preprocess: autoPreprocess(),
            }),

            // If you have external dependencies installed from
            // npm, you'll most likely need these plugins. In
            // some cases you'll need additional configuration -
            // consult the documentation for details:
            // https://github.com/rollup/plugins/tree/master/packages/commonjs
            resolve({
              browser: true,
              dedupe: ["svelte"],
            }),
            commonjs(),

            // In dev mode, call `npm run start` once
            // the bundle has been generated
            !production && serve(),

            // Watch the `public` directory and refresh the
            // browser on changes when not in production
            !production && livereload("public"),
            !production && livereload("src"),

            // If we're building for production (npm run build
            // instead of npm run dev), minify
            production && terser(),
          ],
          watch: {
            clearScreen: false,
          },
        },
      ]
    : [];

export default [
  ...rollupConfigFor("liquid-line"),
  ...rollupConfigFor("demo", true),
];

function serve() {
  let started = false;

  return {
    writeBundle() {
      if (!started) {
        started = true;

        require("child_process").spawn("npm", ["run", "start", "--", "--dev"], {
          stdio: ["ignore", "inherit", "inherit"],
          shell: true,
        });
      }
    },
  };
}
